@@ -93,16 +93,21 @@
     target_os = "dragonfly",
     target_os = "freebsd",
     target_os = "openbsd",
-    target_os = "netbsd"
+    target_os = "netbsd",
+    target_arch = "wasm32",
 ))]
 pub mod unix;
+#[cfg(target_os = "linux")]
+use std::time::Duration;
+
 #[cfg(any(
     target_os = "linux",
     target_os = "macos",
     target_os = "dragonfly",
     target_os = "freebsd",
     target_os = "openbsd",
-    target_os = "netbsd"
+    target_os = "netbsd",
+    target_arch = "wasm32",
 ))]
 pub use unix::*;
 
@@ -112,12 +117,18 @@ pub mod windows;
 pub use windows::*;
 
 /// A error type
-#[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub enum Error {
     /// A value which describes why it is impossible to use such a priority.
     Priority(&'static str),
+    /// Indicates that the priority isn't in range and it should be within the provided range.
+    /// This may happen on different operating systems following a single standard of API but
+    /// allowing different priority values for different scheduling policies.
+    PriorityNotInRange(std::ops::RangeInclusive<i32>),
     /// Target OS' error type. In most systems it is an integer which
     /// later should be used with target OS' API for understanding the value.
+    /// On Linux there is an integer containing an error code from errno.
+    /// For Windows it contains a number used in Windows for the same purpose.
     OS(i32),
     /// FFI failure.
     Ffi(&'static str),
@@ -212,8 +223,25 @@ variant.
     /// Holds scheduling parameters for Deadline scheduling. These are, in order,
     /// the nanoseconds for runtime, deadline, and period. Please note that the
     /// kernel enforces runtime <= deadline <= period.
+    ///
+    ///   arrival/wakeup                    absolute deadline
+    ///        |    start time                    |
+    ///        |        |                         |
+    ///        v        v                         v
+    ///   -----x--------xooooooooooooooooo--------x--------x---
+    ///                 |<-- Runtime ------->|
+    ///        |<----------- Deadline ----------->|
+    ///        |<-------------- Period ------------------->|
     #[cfg(target_os = "linux")]
-    Deadline(u64, u64, u64),
+    Deadline {
+        /// Set this to something larger than the average computation time
+        /// or to the worst-case computation time for hard real-time tasks.
+        runtime: Duration,
+        /// Set this to the relative deadline.
+        deadline: Duration,
+        /// Set this to the period of the task.
+        period: Duration,
+    },
     /// Holds a value representing the maximum possible priority.
     /// Should be used with caution, it solely depends on the target
     /// os where the program is going to be running on, how it will
