@@ -6,8 +6,6 @@
 
 use std::convert::TryFrom;
 
-use const_format::concatcp;
-
 use crate::{Error, ThreadPriority, ThreadPriorityValue};
 use std::mem::MaybeUninit;
 
@@ -19,13 +17,10 @@ const SCHED_FIFO: i32 = 4;
 // Processes scheduled under one of the real-time policies
 // (SCHED_FIFO, SCHED_RR) have a sched_priority value in the range 1
 // (low) to 99 (high).
-const MIN_PRIORITY: i32 = 1;
-const MAX_PRIORITY: i32 = 99;
 // For threads scheduled under one of the normal scheduling policies
 //  (SCHED_OTHER, SCHED_IDLE, SCHED_BATCH), sched_priority is not
 //  used in scheduling decisions (it must be specified as 0).
 // <https://man7.org/linux/man-pages/man7/sched.7.html>
-const NORMAL_PRIORITY: i32 = 0;
 
 /// An alias type for a thread id.
 pub type ThreadId = libc::pthread_t;
@@ -89,7 +84,7 @@ pub enum RealtimeThreadSchedulePolicy {
     /// A deadline policy. Note, due to Linux expecting a pid_t and not a pthread_t, the given
     /// [ThreadId](struct.ThreadId) will be interpreted as a pid_t. This policy is NOT
     /// POSIX-compatible, so we only include it for linux targets.
-    #[cfg(target_os = "linux")]
+    #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
     Deadline,
 }
 
@@ -98,7 +93,7 @@ impl RealtimeThreadSchedulePolicy {
         match self {
             RealtimeThreadSchedulePolicy::Fifo => SCHED_FIFO,
             RealtimeThreadSchedulePolicy::RoundRobin => 2,
-            #[cfg(target_os = "linux")]
+            #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
             RealtimeThreadSchedulePolicy::Deadline => 6,
         }
     }
@@ -108,12 +103,15 @@ impl RealtimeThreadSchedulePolicy {
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum NormalThreadSchedulePolicy {
     /// For running very low priority background jobs
+    #[cfg(not(target_os = "macos"))]
     Idle,
     /// For "batch" style execution of processes
+    #[cfg(not(target_os = "macos"))]
     Batch,
     /// The standard round-robin time-sharing policy
     Other,
     /// The standard round-robin time-sharing policy
+    #[cfg(not(target_os = "macos"))]
     Normal,
 }
 impl NormalThreadSchedulePolicy {
@@ -130,7 +128,6 @@ impl NormalThreadSchedulePolicy {
     fn to_posix(self) -> libc::c_int {
         match self {
             NormalThreadSchedulePolicy::Other => 1,
-            _ => panic!("Invalid value for berkley schedule policy."),
         }
     }
 }
@@ -169,7 +166,7 @@ impl ThreadSchedulePolicy {
             2 => Ok(ThreadSchedulePolicy::Realtime(
                 RealtimeThreadSchedulePolicy::RoundRobin,
             )),
-            #[cfg(target_os = "linux")]
+            #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
             6 => Ok(ThreadSchedulePolicy::Realtime(
                 RealtimeThreadSchedulePolicy::Deadline,
             )),
@@ -197,79 +194,101 @@ impl ThreadSchedulePolicy {
 }
 
 impl ThreadPriority {
+    /// Returns the maximum allowed value for using with the provided policy.
+    /// The returned number is in the range of allowed values.
+    pub fn max_value_for_policy(policy: ThreadSchedulePolicy) -> Result<libc::c_int, Error> {
+        let max_priority = unsafe { libc::sched_get_priority_max(policy.to_posix()) };
+        if max_priority < 0 {
+            #[cfg(not(target_os = "macos"))]
+            unsafe {
+                Err(Error::OS(*libc::__errno_location()))
+            }
+            #[cfg(target_os = "macos")]
+            Err(Error::Priority(
+                "Couldn't get the minimum allowed value of priority for the specified policy",
+            ))
+        } else {
+            Ok(max_priority)
+        }
+    }
+
+    /// Returns the minimum allowed value for using with the provided policy.
+    /// The returned number is in the range of allowed values.
+    pub fn min_value_for_policy(policy: ThreadSchedulePolicy) -> Result<libc::c_int, Error> {
+        let min_priority = unsafe { libc::sched_get_priority_min(policy.to_posix()) };
+        if min_priority < 0 {
+            #[cfg(not(target_os = "macos"))]
+            unsafe {
+                Err(Error::OS(*libc::__errno_location()))
+            }
+            #[cfg(target_os = "macos")]
+            Err(Error::Priority(
+                "Couldn't get the minimum allowed value of priority for the specified policy",
+            ))
+        } else {
+            Ok(min_priority)
+        }
+    }
+
+    /// Checks that the passed priority value is within the range of allowed values for using with the provided policy.
+    pub fn to_allowed_value_for_policy(
+        priority: libc::c_int,
+        policy: ThreadSchedulePolicy,
+    ) -> Result<libc::c_int, Error> {
+        let min_priority = Self::min_value_for_policy(policy)?;
+        let max_priority = Self::max_value_for_policy(policy)?;
+        let allowed_range = min_priority..=max_priority;
+        if allowed_range.contains(&priority) {
+            Ok(priority)
+        } else {
+            Err(Error::PriorityNotInRange(allowed_range))
+        }
+    }
+
+    /// Converts the priority stored to a posix number.
     /// POSIX value can not be known without knowing the scheduling policy
     /// <https://linux.die.net/man/2/sched_get_priority_max>
     pub fn to_posix(self, policy: ThreadSchedulePolicy) -> Result<libc::c_int, Error> {
         let ret = match self {
             ThreadPriority::Min => match policy {
                 // SCHED_DEADLINE doesn't really have a notion of priority, this is an error
-                #[cfg(target_os = "linux")]
+                #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_) => Ok(MIN_PRIORITY as u32),
-                _ => Ok(NORMAL_PRIORITY as u32),
+                _ => Self::min_value_for_policy(policy).map(|v| v as u32),
             },
             ThreadPriority::Crossplatform(ThreadPriorityValue(p)) => match policy {
                 // SCHED_DEADLINE doesn't really have a notion of priority, this is an error
-                #[cfg(target_os = "linux")]
+                #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_)
-                    if !((MIN_PRIORITY..=MAX_PRIORITY).contains(&(p as i32))) =>
-                {
-                    Err(Error::Priority(concatcp!(
-                        "The value is out of range [",
-                        MIN_PRIORITY,
-                        "; ",
-                        MAX_PRIORITY,
-                        "]"
-                    )))
-                }
-                ThreadSchedulePolicy::Normal(_) if p as i32 != NORMAL_PRIORITY => Err(
-                    Error::Priority("The value can be only 0 for normal scheduling policy"),
-                ),
-                _ => Ok(p as u32),
+                _ => Self::to_allowed_value_for_policy(p as i32, policy).map(|v| v as u32),
             },
             // TODO avoid code duplication.
             ThreadPriority::Os(crate::ThreadPriorityOsValue(p)) => match policy {
                 // SCHED_DEADLINE doesn't really have a notion of priority, this is an error
-                #[cfg(target_os = "linux")]
+                #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_)
-                    if !((MIN_PRIORITY..=MAX_PRIORITY).contains(&(p as i32))) =>
-                {
-                    Err(Error::Priority(concatcp!(
-                        "The value is out of range [",
-                        MIN_PRIORITY,
-                        "; ",
-                        MAX_PRIORITY,
-                        "]"
-                    )))
-                }
-                ThreadSchedulePolicy::Normal(_) if p as i32 != NORMAL_PRIORITY => {
-                    Err(Error::Priority(concatcp!(
-                        "The value can be only ",
-                        NORMAL_PRIORITY,
-                        " for normal scheduling policy"
-                    )))
-                }
-                _ => Ok(p),
+                _ => Self::to_allowed_value_for_policy(p as i32, policy).map(|v| v as u32),
             },
             ThreadPriority::Max => match policy {
                 // SCHED_DEADLINE doesn't really have a notion of priority, this is an error
-                #[cfg(target_os = "linux")]
+                #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_) => Ok(MAX_PRIORITY as u32),
-                _ => Ok(NORMAL_PRIORITY as u32),
+                _ => Self::max_value_for_policy(policy).map(|v| v as u32),
             },
-            #[cfg(target_os = "linux")]
-            ThreadPriority::Deadline(_, _, _) => Err(Error::Priority(
+            #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))]
+            ThreadPriority::Deadline {
+                runtime: _,
+                deadline: _,
+                period: _,
+            } => Err(Error::Priority(
                 "Deadline is non-POSIX and cannot be converted.",
             )),
         };
@@ -292,54 +311,111 @@ impl ThreadPriority {
 ///
 /// Setting thread priority to minimum with normal schedule policy:
 ///
-#[cfg_attr(
-    target_os = "macos",
-    doc = "\
-```rust
-use thread_priority::*;
-
-let thread_id = thread_native_id();
-assert!(set_thread_priority_and_policy(thread_id,
-                                       ThreadPriority::Min,
-                                       ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Other)).is_ok());
-```
-"
-)]
-#[cfg_attr(
-    not(target_os = "macos"),
-    doc = "\
-```rust
-use thread_priority::*;
-
-let thread_id = thread_native_id();
-assert!(set_thread_priority_and_policy(thread_id,
-                                       ThreadPriority::Min,
-                                       ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal)).is_ok());
-```
-"
-)]
+/// ```rust
+/// use thread_priority::*;
+///
+/// let thread_id = thread_native_id();
+/// assert!(set_thread_priority_and_policy(thread_id,
+///                                        ThreadPriority::Min,
+///                                        ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Fifo)).is_ok());
+/// ```
+///
+/// # Note
+///
+/// In case the value is specified as [`ThreadPriority::Crossplatform`] and is incompatible with the policy, an error is returned.
+/// However if [`ThreadPriority::Min`] or [`ThreadPriority::Max`] are used, the correct value is used automatically according
+/// to the range of the policy's allowed values.
 pub fn set_thread_priority_and_policy(
     native: ThreadId,
     priority: ThreadPriority,
     policy: ThreadSchedulePolicy,
 ) -> Result<(), Error> {
-    let params = ScheduleParams {
-        sched_priority: match policy {
-            #[cfg(not(target_os = "macos"))]
-            ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => 0,
-            _ => priority.to_posix(policy)?,
-        },
+    let ret = match policy {
+        // SCHED_DEADLINE policy requires its own syscall
+        #[cfg(target_os = "linux")]
+        ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => {
+            use std::convert::TryInto as _;
+            let (runtime, deadline, period) = match priority {
+                ThreadPriority::Deadline {
+                    runtime,
+                    deadline,
+                    period,
+                } => (|| {
+                    Ok((
+                        runtime.as_nanos().try_into()?,
+                        deadline.as_nanos().try_into()?,
+                        period.as_nanos().try_into()?,
+                    ))
+                })()
+                .map_err(|_: std::num::TryFromIntError| {
+                    Error::Priority("Deadline policy durations don't fit into a `u64`.")
+                })?,
+                _ => {
+                    return Err(Error::Priority(
+                        "Deadline policy given without deadline priority.",
+                    ))
+                }
+            };
+            let tid = native as libc::pid_t;
+            let sched_attr = SchedAttr {
+                size: std::mem::size_of::<SchedAttr>() as u32,
+                sched_policy: policy.to_posix() as u32,
+
+                sched_runtime: runtime,
+                sched_deadline: deadline,
+                sched_period: period,
+
+                ..Default::default()
+            };
+            unsafe {
+                libc::syscall(
+                    libc::SYS_sched_setattr,
+                    tid,
+                    &sched_attr as *const _,
+                    // we are not setting SCHED_FLAG_RECLAIM nor SCHED_FLAG_DL_OVERRUN
+                    0,
+                ) as i32
+            }
+        }
+        _ => {
+            let fixed_priority = priority.to_posix(policy)?;
+            let params = ScheduleParams {
+                sched_priority: fixed_priority,
+            }
+            .into_posix();
+            unsafe {
+                libc::pthread_setschedparam(
+                    native,
+                    policy.to_posix(),
+                    &params as *const libc::sched_param,
+                )
+            }
+        }
     };
-    set_thread_schedule_policy(native, policy, params, priority)
+    match ret {
+        0 => Ok(()),
+        e => Err(Error::OS(e)),
+    }
 }
 
 /// Set current thread's priority.
+/// To make the priority of use, the scheduling policy is determined from the value.
+/// If the value is not equal to `ThreadPriority::Crossplatform(0)`, then the one of
+/// the available real-time policies is used. Otherwise, the function does nothing,
+/// as the only reasonable outcome with zero-priority set would be to change a scheduling
+/// policy, for which there is [`set_thread_priority_and_policy`].
+///
+/// * May require privileges
+///
+/// ```rust
+/// use thread_priority::*;
+///
+/// let thread_id = thread_native_id();
+/// assert!(set_current_thread_priority(ThreadPriority::Min).is_ok());
+/// ```
 pub fn set_current_thread_priority(priority: ThreadPriority) -> Result<(), Error> {
     let thread_id = thread_native_id();
-    #[cfg(target_os = "macos")]
-    let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Other);
-    #[cfg(not(target_os = "macos"))]
-    let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal);
+    let policy = ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Fifo);
     set_thread_priority_and_policy(thread_id, priority, policy)
 }
 
@@ -356,76 +432,6 @@ pub fn thread_schedule_policy() -> Result<ThreadSchedulePolicy, Error> {
     thread_schedule_policy_param(thread_native_id()).map(|policy| policy.0)
 }
 
-/// Sets thread schedule policy.
-///
-/// * May require privileges
-/// * Deadline policy requires a tid, not a pthread_t, so invoking this while using a deadline
-/// policy will interpret the given [ThreadId](struct.ThreadId) as a pid_t (thread tid).
-///
-/// # Usage
-/// ```rust,no_run
-/// use thread_priority::*;
-///
-/// let thread_id = thread_native_id();
-/// let policy = ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Fifo);
-/// let params = ScheduleParams { sched_priority: 3 as libc::c_int };
-/// let priority = ThreadPriority::Min;
-/// assert!(set_thread_schedule_policy(thread_id, policy, params, priority).is_ok());
-/// ```
-pub fn set_thread_schedule_policy(
-    native: ThreadId,
-    policy: ThreadSchedulePolicy,
-    params: ScheduleParams,
-    priority: ThreadPriority,
-) -> Result<(), Error> {
-    let params = params.into_posix();
-    unsafe {
-        let ret = match policy {
-            // SCHED_DEADLINE policy requires its own syscall
-            #[cfg(target_os = "linux")]
-            ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => {
-                let (runtime, deadline, period) = match priority {
-                    ThreadPriority::Deadline(r, d, p) => (r, d, p),
-                    _ => {
-                        return Err(Error::Priority(
-                            "Deadline policy given without deadline priority.",
-                        ))
-                    }
-                };
-                let tid = native as libc::pid_t;
-                let sched_attr = SchedAttr {
-                    size: std::mem::size_of::<SchedAttr>() as u32,
-                    sched_policy: policy.to_posix() as u32,
-
-                    sched_runtime: runtime as u64,
-                    sched_deadline: deadline as u64,
-                    sched_period: period as u64,
-
-                    ..Default::default()
-                };
-                libc::syscall(
-                    libc::SYS_sched_setattr,
-                    tid,
-                    &sched_attr as *const _,
-                    // we are not setting SCHED_FLAG_RECLAIM nor SCHED_FLAG_DL_OVERRUN
-                    0,
-                ) as i32
-            }
-            _ => libc::pthread_setschedparam(
-                native,
-                policy.to_posix(),
-                &params as *const libc::sched_param,
-            ),
-        };
-        // This is just to silent the unused variable warning.
-        let _priority = priority;
-        match ret {
-            0 => Ok(()),
-            e => Err(Error::OS(e)),
-        }
-    }
-}
-
 /// Returns policy parameters (schedule policy and other schedule parameters)
 ///
 /// # Usage
@@ -521,20 +527,23 @@ pub trait ThreadExt {
     }
 
     /// Sets current thread's schedule policy.
-    /// For more info read [`set_thread_schedule_policy`].
-    fn set_schedule_policy(
+    /// For more info read [`set_thread_priority_and_policy`].
+    fn set_priority_and_policy(
         &self,
         policy: ThreadSchedulePolicy,
         priority: ThreadPriority,
     ) -> Result<(), Error> {
-        let params = ScheduleParams {
-            sched_priority: match policy {
-                #[cfg(not(target_os = "macos"))]
-                ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => 0,
-                _ => priority.to_posix(policy)?,
-            },
-        };
-        set_thread_schedule_policy(thread_native_id(), policy, params, priority)
+        cfg_if::cfg_if! {
+            if #[cfg(all(target_os = "linux", not(target_arch = "wasm32")))] {
+                if policy == ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) {
+                    set_thread_priority_and_policy(thread_native_id(), ThreadPriority::Crossplatform(ThreadPriorityValue(0)), policy)
+                } else {
+                    set_thread_priority_and_policy(thread_native_id(), priority, policy)
+                }
+            } else {
+                set_thread_priority_and_policy(thread_native_id(), priority, policy)
+            }
+        }
     }
 
     /// Returns native unix thread id.
@@ -590,33 +599,20 @@ mod tests {
         assert!(thread_schedule_policy_param(thread_id).is_ok());
     }
 
-    #[test]
-    fn set_thread_priority_test() {
-        let thread_id = thread_native_id();
-        #[cfg(not(target_os = "macos"))]
-        let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal);
-        #[cfg(target_os = "macos")]
-        let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Other);
-
-        assert!(set_thread_priority_and_policy(thread_id, ThreadPriority::Min, policy,).is_ok());
-        assert!(set_thread_priority_and_policy(thread_id, ThreadPriority::Max, policy,).is_ok());
-        assert!(set_thread_priority_and_policy(
-            thread_id,
-            ThreadPriority::Crossplatform(ThreadPriorityValue(0)),
-            policy,
-        )
-        .is_ok());
-    }
-
     #[test]
     #[cfg(target_os = "linux")]
     fn set_deadline_policy() {
         // allow the identity operation for clarity
         #![allow(clippy::identity_op)]
+        use std::time::Duration;
 
         assert!(set_thread_priority_and_policy(
             0, // current thread
-            ThreadPriority::Deadline(1 * 10_u64.pow(6), 10 * 10_u64.pow(6), 100 * 10_u64.pow(6)),
+            ThreadPriority::Deadline {
+                runtime: Duration::from_millis(1),
+                deadline: Duration::from_millis(10),
+                period: Duration::from_millis(100),
+            },
             ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline)
         )
         .is_ok());
